@@ -320,21 +320,44 @@ class KeycloakProvider(AuthProvider):
     async def get_user(self, user_id: str) -> User:
         """Get user by ID."""
         keycloak_user = await self.admin.get_user(user_id)
-        return self._map_keycloak_user(keycloak_user)
+        # Fetch user's roles
+        try:
+            user_roles = await self.admin.get_user_realm_roles(user_id)
+            role_names = [role.get("name", "") for role in user_roles if role.get("name")]
+        except Exception as e:
+            logger.warning(f"Failed to fetch roles for user {user_id}: {e}")
+            role_names = []
+        return self._map_keycloak_user(keycloak_user, roles=role_names)
 
     async def get_user_by_username(self, username: str) -> User:
         """Get user by username."""
         keycloak_user = await self.admin.get_user_by_username(username)
         if not keycloak_user:
             raise UserNotFoundError(f"User not found: {username}")
-        return self._map_keycloak_user(keycloak_user)
+        user_id = keycloak_user["id"]
+        # Fetch user's roles
+        try:
+            user_roles = await self.admin.get_user_realm_roles(user_id)
+            role_names = [role.get("name", "") for role in user_roles if role.get("name")]
+        except Exception as e:
+            logger.warning(f"Failed to fetch roles for user {user_id}: {e}")
+            role_names = []
+        return self._map_keycloak_user(keycloak_user, roles=role_names)
 
     async def get_user_by_email(self, email: str) -> User:
         """Get user by email."""
         keycloak_user = await self.admin.get_user_by_email(email)
         if not keycloak_user:
             raise UserNotFoundError(f"User not found: {email}")
-        return self._map_keycloak_user(keycloak_user)
+        user_id = keycloak_user["id"]
+        # Fetch user's roles
+        try:
+            user_roles = await self.admin.get_user_realm_roles(user_id)
+            role_names = [role.get("name", "") for role in user_roles if role.get("name")]
+        except Exception as e:
+            logger.warning(f"Failed to fetch roles for user {user_id}: {e}")
+            role_names = []
+        return self._map_keycloak_user(keycloak_user, roles=role_names)
 
     async def list_users(self, filters: UserFilters) -> PaginatedResponse:
         """List users with pagination and filters."""
@@ -353,8 +376,20 @@ class KeycloakProvider(AuthProvider):
             **query,
         )
 
-        # Map to User models
-        users = [self._map_keycloak_user(u) for u in keycloak_users]
+        # Map to User models and fetch roles for each user
+        users = []
+        for keycloak_user in keycloak_users:
+            user_id = keycloak_user["id"]
+            # Fetch user's roles
+            try:
+                user_roles = await self.admin.get_user_realm_roles(user_id)
+                role_names = [role.get("name", "") for role in user_roles if role.get("name")]
+                logger.debug(f"[LIST_USERS] User {keycloak_user.get('username')} has roles: {role_names}")
+            except Exception as e:
+                logger.warning(f"[LIST_USERS] Failed to fetch roles for user {user_id}: {e}")
+                role_names = []
+
+            users.append(self._map_keycloak_user(keycloak_user, roles=role_names))
 
         # Note: Keycloak doesn't provide total count easily, estimate it
         total = len(users) if len(users) < filters.page_size else offset + len(users) + 1
@@ -736,12 +771,28 @@ class KeycloakProvider(AuthProvider):
         )
 
     async def update_role(self, role_id: str, role_data: RoleUpdate) -> Role:
-        """Update role."""
-        # Note: Keycloak roles are primarily name and description
-        # More complex updates would require additional API calls
-        logger.warning("Role update partially implemented for Keycloak")
-        keycloak_role = await self.admin.get_realm_role(role_id)
-        return self._map_keycloak_role(keycloak_role)
+        """Update role with description and/or permissions."""
+        logger.info(f"[PROVIDER] Updating role {role_id}")
+
+        # Build update payload
+        update_payload = {}
+
+        if role_data.description is not None:
+            update_payload["description"] = role_data.description
+
+        # Store permissions in role attributes
+        if role_data.permissions is not None:
+            update_payload["attributes"] = {
+                "permissions": role_data.permissions
+            }
+
+        logger.debug(f"[PROVIDER] Update payload: {update_payload}")
+
+        # Update the role
+        updated_role = await self.admin.update_realm_role(role_id, update_payload)
+        logger.info(f"[PROVIDER] Role {role_id} updated successfully")
+
+        return self._map_keycloak_role(updated_role)
 
     async def delete_role(self, role_id: str) -> bool:
         """Delete role."""
@@ -750,7 +801,9 @@ class KeycloakProvider(AuthProvider):
 
     async def assign_role_to_user(self, user_id: str, role_id: str, scope_id: Optional[str] = None) -> bool:
         """Assign role to user."""
+        logger.info(f"[PROVIDER] Assigning role '{role_id}' to user '{user_id}'")
         await self.admin.assign_realm_roles_to_user(user_id, [role_id])
+        logger.info(f"[PROVIDER] Role '{role_id}' assigned successfully to user '{user_id}'")
         return True
 
     async def remove_role_from_user(self, user_id: str, role_id: str, scope_id: Optional[str] = None) -> bool:
@@ -779,7 +832,7 @@ class KeycloakProvider(AuthProvider):
 
     # ==================== Helper Methods ====================
 
-    def _map_keycloak_user(self, keycloak_user: Dict[str, Any]) -> User:
+    def _map_keycloak_user(self, keycloak_user: Dict[str, Any], roles: Optional[List[str]] = None) -> User:
         """Map Keycloak user to AuthFlow User model."""
         attributes = keycloak_user.get("attributes", {})
 
@@ -797,6 +850,7 @@ class KeycloakProvider(AuthProvider):
             created_at=datetime.fromtimestamp(keycloak_user.get("createdTimestamp", 0) / 1000),
             updated_at=datetime.now(),
             custom_attributes=attributes,
+            roles=roles or [],
         )
 
     def _map_keycloak_group_to_organization(self, keycloak_group: Dict[str, Any]) -> Organization:
@@ -840,6 +894,18 @@ class KeycloakProvider(AuthProvider):
 
     def _map_keycloak_role(self, keycloak_role: Dict[str, Any], scope: Optional[str] = None, scope_id: Optional[str] = None) -> Role:
         """Map Keycloak role to Role."""
+        # Extract permissions from role attributes
+        permissions = []
+        if "attributes" in keycloak_role and keycloak_role["attributes"]:
+            # Permissions are stored as a list in attributes
+            if "permissions" in keycloak_role["attributes"]:
+                perm_attr = keycloak_role["attributes"]["permissions"]
+                # Handle both list and single value
+                if isinstance(perm_attr, list):
+                    permissions = perm_attr
+                else:
+                    permissions = [perm_attr] if perm_attr else []
+
         return Role(
             id=keycloak_role.get("id", keycloak_role["name"]),
             name=keycloak_role["name"],
@@ -848,5 +914,6 @@ class KeycloakProvider(AuthProvider):
             scope=scope or "global",
             scope_id=scope_id,
             is_composite=keycloak_role.get("composite", False),
+            permissions=permissions,
             created_at=datetime.now(),
         )
