@@ -555,21 +555,77 @@ class KeycloakAdminClient:
         except Exception as e:
             self._handle_error(e, f"create_realm_role({name})")
 
-    async def get_realm_role(self, role_name: str) -> Dict[str, Any]:
-        """Get realm role by name.
+    async def get_realm_role(self, role_identifier: str) -> Dict[str, Any]:
+        """Get realm role by name or ID.
 
         Args:
-            role_name: Role name
+            role_identifier: Role name or role ID (UUID)
 
         Returns:
             Role data
         """
         try:
             admin = self._get_admin()
-            role = admin.get_realm_role(role_name)
-            return role
+            # First try to get by name
+            try:
+                logger.debug(f"[KEYCLOAK_ADMIN] Attempting to get role by name: {role_identifier}")
+                role = admin.get_realm_role(role_identifier)
+                logger.info(f"[KEYCLOAK_ADMIN] Found role by name: {role.get('name')} (ID: {role.get('id')})")
+                return role
+            except KeycloakGetError:
+                # If not found by name, it might be an ID - get all roles and find by ID
+                logger.debug(f"[KEYCLOAK_ADMIN] Role not found by name, searching by ID: {role_identifier}")
+                all_roles = admin.get_realm_roles()
+                role = next((r for r in all_roles if r.get("id") == role_identifier), None)
+                if role:
+                    logger.info(f"[KEYCLOAK_ADMIN] Found role by ID: {role.get('name')} (ID: {role.get('id')})")
+                    return role
+                else:
+                    logger.error(f"[KEYCLOAK_ADMIN] Role not found by name or ID: {role_identifier}")
+                    raise RoleNotFoundError(f"Role not found: {role_identifier}")
         except Exception as e:
-            self._handle_error(e, f"get_realm_role({role_name})")
+            self._handle_error(e, f"get_realm_role({role_identifier})")
+
+    async def update_realm_role(self, role_identifier: str, role_data: Dict[str, Any]) -> Dict[str, Any]:
+        """Update realm role.
+
+        Args:
+            role_identifier: Role name or ID (UUID)
+            role_data: Role update data (description, attributes, etc.)
+
+        Returns:
+            Updated role data
+        """
+        try:
+            admin = self._get_admin()
+
+            # First get the role to find its name (Keycloak update requires role name)
+            role = await self.get_realm_role(role_identifier)
+            role_name = role.get("name")
+
+            logger.info(f"[KEYCLOAK_ADMIN] Updating role: {role_name} (ID: {role.get('id')})")
+            logger.debug(f"[KEYCLOAK_ADMIN] Update data: {role_data}")
+
+            # Update role using Keycloak's update_realm_role method
+            # Note: Keycloak expects the full payload, so we merge with existing data
+            update_payload = {
+                "name": role_name,  # Name is required and cannot be changed
+                "description": role_data.get("description", role.get("description", "")),
+            }
+
+            # Add attributes if provided
+            if "attributes" in role_data:
+                update_payload["attributes"] = role_data["attributes"]
+            elif "attributes" in role:
+                update_payload["attributes"] = role.get("attributes", {})
+
+            admin.update_realm_role(role_name, update_payload)
+            logger.info(f"[KEYCLOAK_ADMIN] Successfully updated role: {role_name}")
+
+            # Return the updated role
+            return await self.get_realm_role(role_name)
+        except Exception as e:
+            self._handle_error(e, f"update_realm_role({role_identifier})")
 
     async def list_realm_roles(self) -> List[Dict[str, Any]]:
         """List all realm roles.
@@ -604,18 +660,41 @@ class KeycloakAdminClient:
 
         Args:
             user_id: User ID
-            role_names: List of role names
+            role_names: List of role names or role IDs (UUIDs)
         """
         try:
             admin = self._get_admin()
+            logger.info(f"[KEYCLOAK_ADMIN] Starting role assignment - User: {user_id}, Roles: {role_names}")
 
             # Get role representations
-            roles = [admin.get_realm_role(name) for name in role_names]
+            # Handle both role names and role IDs (UUIDs)
+            roles = []
+            for role_identifier in role_names:
+                try:
+                    # First try to get by name
+                    logger.debug(f"[KEYCLOAK_ADMIN] Attempting to get role by name: {role_identifier}")
+                    role = admin.get_realm_role(role_identifier)
+                    logger.info(f"[KEYCLOAK_ADMIN] Found role by name: {role.get('name')} (ID: {role.get('id')})")
+                    roles.append(role)
+                except KeycloakGetError:
+                    # If not found by name, it might be an ID - get all roles and find by ID
+                    logger.debug(f"[KEYCLOAK_ADMIN] Role not found by name, searching by ID: {role_identifier}")
+                    all_roles = admin.get_realm_roles()
+                    role = next((r for r in all_roles if r.get("id") == role_identifier), None)
+                    if role:
+                        logger.info(f"[KEYCLOAK_ADMIN] Found role by ID: {role.get('name')} (ID: {role.get('id')})")
+                        roles.append(role)
+                    else:
+                        logger.error(f"[KEYCLOAK_ADMIN] Role not found by name or ID: {role_identifier}")
+                        raise RoleNotFoundError(f"Role not found: {role_identifier}")
 
             # Assign roles
+            logger.info(f"[KEYCLOAK_ADMIN] Calling Keycloak API to assign {len(roles)} role(s) to user {user_id}")
+            logger.debug(f"[KEYCLOAK_ADMIN] Roles to assign: {[r.get('name') for r in roles]}")
             admin.assign_realm_roles(user_id, roles)
-            logger.info(f"Assigned roles {role_names} to user {user_id}")
+            logger.info(f"[KEYCLOAK_ADMIN] Successfully assigned roles {role_names} to user {user_id}")
         except Exception as e:
+            logger.error(f"[KEYCLOAK_ADMIN] Error assigning roles: {e}", exc_info=True)
             self._handle_error(e, f"assign_realm_roles_to_user({user_id})")
 
     async def get_user_realm_roles(self, user_id: str) -> List[Dict[str, Any]]:
@@ -641,13 +720,27 @@ class KeycloakAdminClient:
 
         Args:
             user_id: User ID
-            role_names: List of role names
+            role_names: List of role names or role IDs (UUIDs)
         """
         try:
             admin = self._get_admin()
 
             # Get role representations
-            roles = [admin.get_realm_role(name) for name in role_names]
+            # Handle both role names and role IDs (UUIDs)
+            roles = []
+            for role_identifier in role_names:
+                try:
+                    # First try to get by name
+                    role = admin.get_realm_role(role_identifier)
+                    roles.append(role)
+                except KeycloakGetError:
+                    # If not found by name, it might be an ID - get all roles and find by ID
+                    all_roles = admin.get_realm_roles()
+                    role = next((r for r in all_roles if r.get("id") == role_identifier), None)
+                    if role:
+                        roles.append(role)
+                    else:
+                        raise RoleNotFoundError(f"Role not found: {role_identifier}")
 
             # Remove roles
             admin.delete_realm_roles_of_user(user_id, roles)
@@ -662,13 +755,27 @@ class KeycloakAdminClient:
 
         Args:
             group_id: Group ID
-            role_names: List of role names
+            role_names: List of role names or role IDs (UUIDs)
         """
         try:
             admin = self._get_admin()
 
             # Get role representations
-            roles = [admin.get_realm_role(name) for name in role_names]
+            # Handle both role names and role IDs (UUIDs)
+            roles = []
+            for role_identifier in role_names:
+                try:
+                    # First try to get by name
+                    role = admin.get_realm_role(role_identifier)
+                    roles.append(role)
+                except KeycloakGetError:
+                    # If not found by name, it might be an ID - get all roles and find by ID
+                    all_roles = admin.get_realm_roles()
+                    role = next((r for r in all_roles if r.get("id") == role_identifier), None)
+                    if role:
+                        roles.append(role)
+                    else:
+                        raise RoleNotFoundError(f"Role not found: {role_identifier}")
 
             # Assign roles to group
             admin.assign_group_realm_roles(group_id, roles)
